package ru.dbotthepony.mc.otm.client.screen.panels.buttons

import com.mojang.blaze3d.platform.InputConstants
import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.client.gui.screens.Screen
import ru.dbotthepony.mc.otm.client.playGuiClickSound
import ru.dbotthepony.mc.otm.client.render.AbstractMatterySprite
import ru.dbotthepony.mc.otm.client.render.WidgetLocation
import ru.dbotthepony.mc.otm.client.screen.panels.EditablePanel

open class CheckBoxPanel<out S : Screen> @JvmOverloads constructor(
	screen: S,
	parent: EditablePanel<*>?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = REGULAR_DIMENSIONS,
	height: Float = REGULAR_DIMENSIONS
) : EditablePanel<S>(screen, parent, x, y, width, height) {
	open var checked = false
	open var isDisabled = false
	protected var isPressed = false

	override fun innerRender(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
		if (isDisabled) {
			if (checked) {
				DISABLED_CHECKED.render(stack, width = width, height = height)
			} else {
				DISABLED_UNCHECKED.render(stack, width = width, height = height)
			}
		} else {
			if (isPressed) {
				if (checked) {
					PRESSED_CHECKED.render(stack, width = width, height = height)
				} else {
					PRESSED_UNCHECKED.render(stack, width = width, height = height)
				}
			} else if (isHovered) {
				if (checked) {
					HOVERED_CHECKED.render(stack, width = width, height = height)
				} else {
					HOVERED_UNCHECKED.render(stack, width = width, height = height)
				}
			} else {
				if (checked) {
					IDLE_CHECKED.render(stack, width = width, height = height)
				} else {
					IDLE_UNCHECKED.render(stack, width = width, height = height)
				}
			}
		}
	}

	protected open fun onClick() {
		checked = !checked
	}

	override fun mouseClickedInner(x: Double, y: Double, button: Int): Boolean {
		if (!isDisabled && button == InputConstants.MOUSE_BUTTON_LEFT) {
			grabMouseInput = true
			isPressed = true
			playGuiClickSound()
		}

		return true
	}

	override fun mouseReleasedInner(x: Double, y: Double, button: Int): Boolean {
		if (isPressed && button == InputConstants.MOUSE_BUTTON_LEFT) {
			if (isHovered) {
				onClick()
			}

			isPressed = false
			grabMouseInput = false
		}

		return true
	}

	companion object {
		const val REGULAR_DIMENSIONS = 15f

		val IDLE_UNCHECKED: AbstractMatterySprite
		val IDLE_CHECKED: AbstractMatterySprite
		val HOVERED_UNCHECKED: AbstractMatterySprite
		val HOVERED_CHECKED: AbstractMatterySprite
		val PRESSED_UNCHECKED: AbstractMatterySprite
		val PRESSED_CHECKED: AbstractMatterySprite
		val DISABLED_UNCHECKED: AbstractMatterySprite
		val DISABLED_CHECKED: AbstractMatterySprite

		init {
			val grid = WidgetLocation.CHECKBOX.grid(columns = 2, rows = 4)

			IDLE_UNCHECKED = grid.next()
			IDLE_CHECKED = grid.next()
			HOVERED_UNCHECKED = grid.next()
			HOVERED_CHECKED = grid.next()
			PRESSED_UNCHECKED = grid.next()
			PRESSED_CHECKED = grid.next()
			DISABLED_UNCHECKED = grid.next()
			DISABLED_CHECKED = grid.next()
		}
	}
}

